//
// Logic for events. These things are responsible for creating
// all sorts of fun stuff, unpredictability, etc.
//
//           NGCollector, Sebastian Mihai, 2012
//

#include "cframework\ngpc.h"
#include "cframework\library.h"

#include "events.h"
#include "globaldefs.h"
#include "collection.h"

// fellow collector will offer trades within this price difference
#define COLLECTOR_OFFER_DIFF_LIMIT 250

// given a game fellow collector wants from the player,
// find a game to trade for it, whose price is within COLLECTOR_OFFER_DIFF_LIMIT
// of the game we want from the player
//
// return 1 if such a game was found
// return 0 if no such game was found
u16 private_GetFellowCollectorOffer( u16 yours, u16* offer ) {
	u16 r, delta;
	u16 found;
	
	// choose any game
	r = GetRandom(GAME_COUNT);
	
	found = 0;
	delta = 0;
	// start looking left and then right of the randomly chosen index until we find a 
	// game whose value is within 50 of the game we want from the player
	while(1) {
		// look to the left
		if( (s16)(r - delta) >= 0 ) {
			// game is within limit
			if( GameValues[r - delta] - GameValues[yours] <= COLLECTOR_OFFER_DIFF_LIMIT 
			 || GameValues[yours] - GameValues[r - delta] <= COLLECTOR_OFFER_DIFF_LIMIT
			) {
				// game is NOT the game we want from the player
				if( r - delta != yours ) {
					found = r - delta;
					break;
				}
			}
		}
		
		// look to the right
		if( r + delta <= GAME_COUNT - 1 ) {
			// game is within limit
			if( GameValues[r + delta] - GameValues[yours] <= COLLECTOR_OFFER_DIFF_LIMIT 
			 || GameValues[yours] - GameValues[r + delta] <= COLLECTOR_OFFER_DIFF_LIMIT
			) {
				// game is NOT the game we want from the player
				if( r + delta != yours ) {
					found = r + delta;
					break;
				}
			}
		}
		
		// did we look through all possible games?
		if( (s16)(r - delta) < 0 && r + delta > GAME_COUNT - 1 ) {
			return 0;
		}
		
		delta++;
	}
	
	*offer = found;
	
	return 1;
}

u16 GetFellowCollectorGames( u16* his, u16* yours ) {
	u16 r, delta;
	u16 found;
	u16 offer;
	
	// choose any game
	r = GetRandom(GAME_COUNT);
	
	found = 0;
	delta = 0;
	// start looking left and then right of the randomly chosen index until we find a 
	// game which the player owns
	while(1) {
		// look to the left
		if( (s16)(r - delta) >= 0 ) {
			if( GameOwn[r - delta] == 1 ) {
				found = r - delta;
				break;
			}
		}
		
		// look to the right
		if( r + delta <= GAME_COUNT - 1 ) {
			if( GameOwn[r + delta] == 1 ) {
				found = r + delta;
				break;
			}
		}
		
		// did we look through all possible games?
		if( (s16)(r - delta) < 0 && r + delta > GAME_COUNT - 1 ) {
			return 0;
		}
		
		delta++;
	}
	
	// we want this one from the player
	*his = found;
	
	found = private_GetFellowCollectorOffer( *his, &offer );
	
	// in order to offer a trade, we have to have found a suitable game
	// but not the SAME game that we want from the player :)
	if( found ) {
		*yours = offer;
		return 1;
	}
	
	return 0;
}



u16 GetRandomItem( s32 maxPrice, u16* itemIndex, u16* price ) {
	u16 r, delta;
	u16 found;
	
	// choose any game
	r = GetRandom(GAME_COUNT);
	
	found = 0;
	delta = 0;
	// start looking left and then right of the randomly chosen index until we find a 
	// game which the player can afford
	while(1) {
		// look to the left
		if( (s16)(r - delta) >= 0 ) {
			if( GameValues[r - delta] <= maxPrice ) {
				found = r - delta;
				break;
			}
		}
		
		// look to the right
		if( r + delta <= GAME_COUNT - 1 ) {
			if( GameValues[r + delta] <= maxPrice ) {
				found = r + delta;
				break;
			}
		}
		
		// did we look through all possible games?
		if( (s16)(r - delta) < 0 && r + delta > GAME_COUNT - 1 ) {
			return 0;
		}
		
		delta++;
	}
	
	// we found a game the player can buy
	*itemIndex = found;
	*price = GameValues[ found ];
	
	return 1;
}

u16 GetGambleResult( u16 amountGambled ) {
	u16 r;

	// shhh :)
	if(amountGambled<20) r=42;
		else r=39;
	
	// did the player win at all?
	if( GetRandom(100) < 50 ) {
		// player won
		
		// to randomly generate how much the player won, use the formula
		//
		// limit = 4*E    where E is the overall expected value, for example 1.1 for 110%
		//
		// then we call GetRandom(limit)
		//
		return GetRandom((amountGambled/10)*r)+1;
	}
	
	// player lost
	return 0;
}

u16 GetRandomPlayerGame( u16* itemIndex ) {
	u16 r, delta;
	u16 found;
	u16 offer;
	
	// choose any game
	r = GetRandom(GAME_COUNT);
	
	found = 0;
	delta = 0;
	// start looking left and then right of the randomly chosen index until we find a 
	// game which the player owns
	while(1) {
		// look to the left
		if( (s16)(r - delta) >= 0 ) {
			if( GameOwn[r - delta] == 1 ) {
				found = r - delta;
				break;
			}
		}
		
		// look to the right
		if( r + delta <= GAME_COUNT - 1 ) {
			if( GameOwn[r + delta] == 1 ) {
				found = r + delta;
				break;
			}
		}
		
		// did we look through all possible games?
		if( (s16)(r - delta) < 0 && r + delta > GAME_COUNT - 1 ) {
			return 0;
		}
		
		delta++;
	}
	
	// we want this one from the player
	*itemIndex = found;
	return 1;
}

u16 GetRandomNotOwnedGame( u16* itemIndex ) {
	u16 r, delta;
	u16 found;
	u16 offer;
	
	// choose any game
	r = GetRandom(GAME_COUNT);
	
	found = 0;
	delta = 0;
	// start looking left and then right of the randomly chosen index until we find a 
	// game which the player owns
	while(1) {
		// look to the left
		if( (s16)(r - delta) >= 0 ) {
			if( GameOwn[r - delta] == 0 ) {
				found = r - delta;
				break;
			}
		}
		
		// look to the right
		if( r + delta <= GAME_COUNT - 1 ) {
			if( GameOwn[r + delta] == 0 ) {
				found = r + delta;
				break;
			}
		}
		
		// did we look through all possible games?
		if( (s16)(r - delta) < 0 && r + delta > GAME_COUNT - 1 ) {
			return 0;
		}
		
		delta++;
	}
	
	// we want this one from the player
	*itemIndex = found;
	return 1;
}
