//
// Handles the game list (what you own) screen and interaction (view only)
//
//           NGCollector, Sebastian Mihai, 2012
//

#include "cframework\ngpc.h"
#include "cframework\library.h"

#include "gamelist.h"
#include "collection.h"
#include "messagearea.h"
#include "globaldefs.h"

#define FIRST_GAME_LIST_LINE 2
#define GAME_LIST_PAGE_SIZE 15

#define DEFAULT_PALETTE 6
#define MISSING_GAME_PALETTE 7
#define OWNED_GAME_PALETTE 8
#define GAME_VALUE_PALETTE 9

// the key that takes us out of this screen
#define EXIT_SCREEN_KEY J_OPTION

// track current page of the games list
// a global variable is used, so that we can easily return to the same page we left off
// when we come back to the games list again
u8 private_CurrentPage = 0;

// clear only the lines used by the game list
void private_ClearListArea() {
	u8 i;
	
	for(i=FIRST_GAME_LIST_LINE; i<TOTAL_TEXT_LINES-FIRST_GAME_LIST_LINE; i++) {
		PrintString(SCR_1_PLANE, 0, 0, i,  "                    ");
	}
}

// clear entire screen and print game list header
void private_InitializeGameList() {
	// clear screen
	FillScreen(SCR_1_PLANE, (u16)' ', 0);
	FillScreen(SCR_2_PLANE, (u16)' ', 0);
	
	SetBackgroundColour(RGB(15, 15, 15));
	
	SetColors(SCR_1_PLANE, DEFAULT_PALETTE, 0, 0, RGB(2, 7, 7));
	SetColors(SCR_1_PLANE, MISSING_GAME_PALETTE, 0, 0, RGB(7, 7, 7));
	SetColors(SCR_1_PLANE, OWNED_GAME_PALETTE, 0, 0, RGB(0, 10, 3));
	SetColors(SCR_1_PLANE, GAME_VALUE_PALETTE, 0, 0, RGB(10, 0, 10));

	// lines 0 and 18 are taken up by instructions
	// lines 1 and 17 are blank
	// lines 2 through 16 are taken up by the games list (15 lines per page)
	
	PrintString(SCR_1_PLANE, DEFAULT_PALETTE, 0, 0,  "** MY COLLECTION! **");
	PrintString(SCR_1_PLANE, DEFAULT_PALETTE, 2, TOTAL_TEXT_LINES-1,  "[OPTION] to exit");
}

// private worker function which displays a page of games from the collection
void private_ListGamesPage() {
	u8 i, digits;
	u8 textPalette;
	u16 currentGameIndex;
	
	private_ClearListArea();
	
	for(i=0; i<GAME_LIST_PAGE_SIZE; i++ ) {
		
		currentGameIndex = private_CurrentPage*GAME_LIST_PAGE_SIZE + i;
		
		// if the counter is past the last game, we're done
		if( currentGameIndex >= GAME_COUNT ) {
			break;
		}
		
		// default to missing
		textPalette = MISSING_GAME_PALETTE;
		
		// do we own this game?
		if( GameOwn[currentGameIndex] != 0 ) {
			textPalette = OWNED_GAME_PALETTE;
		}
		
		// write game name
		PrintString(SCR_1_PLANE, textPalette, 0, FIRST_GAME_LIST_LINE+i,  GameNames[currentGameIndex] );
		
		// write game value		
		digits = CalculateDigits(GameValues[ currentGameIndex ]);
		PrintDecimal(SCR_1_PLANE, GAME_VALUE_PALETTE, 20-digits, FIRST_GAME_LIST_LINE+i, GameValues[ currentGameIndex ], digits );
	}
}

//
// Entry point to the game list screen
//
// UP, LEFT - previous page
// DOWN, RIGHT - next page
// OPTION - exit
//
void ShowGameList() {
	
	// initialize screen, colours, etc.
	private_InitializeGameList();
	
	// initial rendering of list - it will remember the page from when it was last closed
	private_ListGamesPage();
	
	// debounce
	while (SYS_LEVER != 0) { WaitVsync(); }
	
	while(1) {
		// wait for input
		while (SYS_LEVER == 0) { WaitVsync(); GetRandom(10000); }
		
		// previous page
		if ( (SYS_LEVER & J_UP || SYS_LEVER & J_LEFT) && private_CurrentPage > 0) {
			private_CurrentPage--;
			private_ListGamesPage();
		}
		
		// next page
		if ( (SYS_LEVER & J_DOWN || SYS_LEVER & J_RIGHT) && private_CurrentPage < (GAME_COUNT / GAME_LIST_PAGE_SIZE ) ) {
			private_CurrentPage++;
			private_ListGamesPage();
		}
		
		// we're out of here if OPTION is pressed
		if( SYS_LEVER & EXIT_SCREEN_KEY ) {
			break;
		}
		
		//debounce (wait for input to be released)
		while (SYS_LEVER != 0) { WaitVsync(); }
	}
	
	//debounce the OPTION
	while (SYS_LEVER != 0) { WaitVsync(); }
}
