//
// Handles the game logic
//
//           NGCollector, Sebastian Mihai, 2012
//

// platform includes
#include "cframework\ngpc.h"
#include "cframework\library.h"

// standard C includes
#include <stdlib.h>
#include <stdio.h>

// NGCOLLECTOR routines
#include "gameplay.h"
#include "globaldefs.h"
#include "collection.h"
#include "messagearea.h"
#include "gamelist.h"
#include "levels.h"
#include "events.h"
#include "graphics.h"

#define PLAYER_PALETTE_TOP 0
#define PLAYER_PALETTE 1
#define NPC_PALETTE 2

const u8 Levels[MAX_LEVELS] = { LEVEL_BURLY_BURGER, LEVEL_FLEA_MARKET, LEVEL_CONVENIENCE_STORE, LEVEL_CASINO, LEVEL_FELLOW_COLLECTOR };

s16 playerX, playerY;
u8 animationFrame;
u8 alreadyAdvancedAnimation; //used for character animation
u8 facing; //0=left, 1=right
s8 currentLevelIndex;
s32 money;
u16 policeChance;//percentage chance cops will catch you if you rob the convenience store

// holds the type of last event
u8 lastEvent;
// data for events
s32 lastEventData[5];

#define EVENT_NONE 0
#define EVENT_FELLOW_COLLECTOR_TRADE 1
#define EVENT_CASINO_GAMBLE 2
#define EVENT_FLEA_MARKET_OFFER 3
#define EVENT_BURLY_BURGER_WORK 4
#define EVENT_BURLY_CONVENIENCE_ROB 5
#define EVENT_FLEA_MARKET_BROKE 6
#define EVENT_CASINO_BROKE 7
#define EVENT_FELLOW_COLLECTOR_NONE 8

void private_PrintMoney() {
	u8 digits;
	// display money
	digits = CalculateDigits( money );
	PrintString(SCR_1_PLANE, 0, 0, 10, "                    ");
	PrintString(SCR_1_PLANE, 0, 20-digits-2, 10, "$");
	PrintDecimal(SCR_1_PLANE, 0, 20-digits, 10, (u16)money, digits );
}

void private_GenerateEvent() {
	u8 level = Levels[currentLevelIndex];
	u16 a, b, c;
	
	if( level == LEVEL_BURLY_BURGER ) {
		lastEvent = EVENT_BURLY_BURGER_WORK;
	}
	
	if( level == LEVEL_FLEA_MARKET ) {
		
		//generate item offered by flea market, along with its price
		a = GetRandomItem( money, &b, &c );
		if( a ) {
			lastEvent = EVENT_FLEA_MARKET_OFFER;
			lastEventData[0] = b;//index
			lastEventData[1] = c;//price
		} else {
			lastEvent = EVENT_FLEA_MARKET_BROKE;
		}
	}
	
	if( level == LEVEL_CONVENIENCE_STORE ) {
		lastEvent = EVENT_BURLY_CONVENIENCE_ROB;
	}
	
	if( level == LEVEL_CASINO ) {
		if( money >= 10 ) {
			lastEvent = EVENT_CASINO_GAMBLE;
			lastEventData[0] = money;
		} else {
			lastEvent = EVENT_CASINO_BROKE;
		}
	}
	
	if( level == LEVEL_FELLOW_COLLECTOR ) {
		
		a = GetFellowCollectorGames( &b, &c );
	
		if( a ) {
			lastEvent = EVENT_FELLOW_COLLECTOR_TRADE;
			lastEventData[0] = b; //player's game
			lastEventData[1] = c; //fellow collector's game
		} else {
			lastEvent = EVENT_FELLOW_COLLECTOR_NONE;
		}
	}
}

void private_HandleBurgerWork() {
	u8 lastKeyWasLeft;
	
	ClearMessageArea();
	AddMessage( (char*)"[LEFT] then [RIGHT]" );
	AddMessage( (char*)"to cook hamburgers" );
	AddMessage( (char*)"and earn" );
	AddAmountMessage( 1 );
	AddMessage( (char*)"[A] to end shift" );
	
	// debounce
	while (SYS_LEVER != 0) { WaitVsync(); }
	
	lastKeyWasLeft = 0;
	
	while(1) {
		
		private_PrintMoney();
		
		if( SYS_LEVER ) {
			if( SYS_LEVER & J_LEFT ) {
				lastKeyWasLeft = 1;
			} else if( SYS_LEVER & J_RIGHT ) {
				// NOTE that we don't go in here if LEFT is still pressed
				// so the player must actually let go of LEFT before pressing RIGHT
				if( lastKeyWasLeft ) {
					money = money + 1;
				}
				lastKeyWasLeft = 0;
				while (SYS_LEVER != 0) { WaitVsync(); }
			} else {
				if( SYS_LEVER & J_A ) {
					break;
				}
				lastKeyWasLeft = 0;
				while (SYS_LEVER != 0) { WaitVsync(); }
			}
		}
		
		WaitVsync();
	}
}

void private_RenderEvent() {
	if( lastEvent == EVENT_BURLY_BURGER_WORK ) {
		AddMessage( (char*)"[A] to start working" );
		AddMessage( (char*)"[OPTION] to view" );
	}
	
	if( lastEvent == EVENT_FLEA_MARKET_OFFER ) {
		AddMessage( (char*)"[A] to buy" );
		AddMessageGame( (char*) GameNames[ lastEventData[0] ] );
		AddMessage( (char*)"for" );
		AddAmountMessage( (u16)lastEventData[1] );
		AddMessage( (char*)"[OPTION] to view" );
	}
	
	if( lastEvent == EVENT_FELLOW_COLLECTOR_TRADE ) {
		AddMessage( (char*)"he wants to trade" );
		AddMessageGame( (char*) GameNames[ lastEventData[1] ] );
		AddMessage( (char*)"for your" );
		AddMessageGame( (char*) GameNames[ lastEventData[0] ] );
		AddMessage( (char*)"[A] to trade" );
		AddMessage( (char*)"[OPTION] to view" );
	}
	
	if( lastEvent == EVENT_CASINO_GAMBLE ) {
		AddMessage( (char*)"[A] to gamble $ 10" );
				
		if( lastEventData[0] >= 25 )
			AddMessage( (char*)"[B] to gamble $ 25" );
		
		AddMessage( (char*)"[OPTION] to view" );
	}
	
	if( lastEvent == EVENT_BURLY_CONVENIENCE_ROB ) {
		AddMessage( (char*)"[A] to rob" );
		AddMessage( (char*)"[OPTION] to view" );
	}
	
	if( lastEvent == EVENT_FLEA_MARKET_BROKE ) {
		AddMessage( (char*)"It seems you cannot" );
		AddMessage( (char*)"afford anything ..." );
		AddMessage( (char*)"[OPTION] to view" );
	}
	
	if( lastEvent == EVENT_CASINO_BROKE ) {
		AddMessage( (char*)"You are too broke" );
		AddMessage( (char*)"to gamble ..." );
		AddMessage( (char*)"[OPTION] to view" );
	}
	
	if( lastEvent == EVENT_FELLOW_COLLECTOR_NONE ) {
		AddMessage( (char*)"None of your games" );
		AddMessage( (char*)"interest him ..." );
		AddMessage( (char*)"[OPTION] to view" );
	}
	
	if( lastEvent == EVENT_NONE ) {
		AddMessage( (char*)"[OPTION] to view" );
	}
}

void private_GenerateRandomEvent() {
	u8 temp;
	u8 eventHandled;
	u16 temp2, temp3;
	
	// FIND A GAME YOU DO NOT HAVE!!!!
	temp2 = GetRandom(100);
	if( temp2 < 2 ) {
		// found a game!!
		temp2 = GetRandomNotOwnedGame( &temp3 );
		if( temp2 ) {
			// add game
			GameOwn[ temp3 ] = 1;
			AddMessage( (char*)"YOU HAVE FOUND" );
			AddMessageGame( (char*) GameNames[ temp3 ] );
			AddMessage( (char*)"ON THE STREET!" );
		}
		return;
	}
	
	// NEO THIEVES!!!
	temp2 = GetRandom(100);
	if( temp2 < 2 ) {
		// caught by Neo Thieves
		temp2 = GetRandomPlayerGame( &temp3 );
		if( temp2 ) {
			// we found a game to steal!
			GameOwn[ temp3 ] = 0;
			AddMessage( (char*)"NEO THIEVES AMBUSHED" );
			AddMessage( (char*)"YOU AND STOLE" );
			AddMessageGame( (char*) GameNames[ temp3 ] );
		}
		return;
	}
}

// handle the A button being pressed
void private_HandleAButton() {
	u8 temp;
	u8 eventHandled;
	u16 temp2, temp3;
	eventHandled = 0;
	if( lastEvent == EVENT_FLEA_MARKET_OFFER ) {
		money = money - lastEventData[1];
		GameOwn[ lastEventData[0] ] = 1;
		AddMessageBlank();
		AddMessage( (char*)"You have purchased" );
		AddMessageGame( (char*) GameNames[ lastEventData[0] ] );
		AddMessage( (char*)"[OPTION] to view" );
		
		// immediately offer a new game if possible
		private_GenerateEvent();
		private_RenderEvent();
		
		eventHandled = 1;
	}
	
	if( lastEvent == EVENT_FELLOW_COLLECTOR_TRADE ) {
		GameOwn[ lastEventData[0] ] = 0;
		GameOwn[ lastEventData[1] ] = 1;
		
		AddMessageBlank();
		AddMessage( (char*)"You obtained" );
		AddMessageGame( (char*) GameNames[ lastEventData[1] ] );
		AddMessage( (char*)"and lost" );
		AddMessageGame( (char*) GameNames[ lastEventData[0] ] );
		AddMessage( (char*)"[OPTION] to view" );
		
		lastEvent = EVENT_NONE;
		eventHandled = 1;
	}
	
	if( lastEvent == EVENT_CASINO_GAMBLE ) {
		temp2 = GetGambleResult(10);
		money = money - 10;
		if( temp2 == 0 ) {
			AddMessage( (char*)"You lost!" );
		} else {
			AddMessage( (char*)"You won" );
			AddAmountMessage( temp2 );
			money = money + temp2;
		}
		AddMessageBlank();
		
		// player can keep gambling
		private_GenerateEvent();
		private_RenderEvent();
		eventHandled = 1;
	}
	
	if( lastEvent == EVENT_BURLY_CONVENIENCE_ROB ) {
		
		AddMessageBlank();
		if( GetRandom(100) < policeChance ) {
			// caught by police
			AddMessage( (char*)"Police caught you" );
			
			temp2 = GetRandomPlayerGame( &temp3 );
			if( temp2 ) {
				// confiscate a game
				GameOwn[ temp3 ] = 0;
				AddMessage( (char*)"and confiscated your" );
				AddMessageGame( (char*) GameNames[ temp3 ] );
			} else {
				// confiscate half the player's money
				AddMessage( (char*)"but found no games" );
				AddMessage( (char*)"and confiscated" );
				temp3 = money / 2;
				money = money - temp3;
				AddAmountMessage( temp3 );
			}
			
		} else {
			// successful
			temp2 = 400 + GetRandom(200);
			AddMessage( (char*)"You looted" );
			AddAmountMessage( temp2 );
			money = money + temp2;
			// every time you rob the place, it becomes more likely that police will catch you
			policeChance = policeChance + 5;
		}
		
		AddMessage( (char*)"[OPTION] to view" );
		eventHandled = 1;
		
		if( policeChance > 80 ) {
			// but caps at a certain percentage
			policeChance = 80;
		}
		
		lastEvent = EVENT_NONE;
	}
	
	if( lastEvent == EVENT_BURLY_BURGER_WORK ) {		
		private_HandleBurgerWork();
		
		// after we stop working, we re-display the text 
		ClearMessageArea();
		AddMessage( (char*)"You see" );
		AddMessage( (char*)"Burly Burger" );
		// player can go back to working
		private_GenerateEvent();
		private_RenderEvent();
		eventHandled = 1;
	}
	
	// debounce if we actually did something
	if( eventHandled ) 
		while (SYS_LEVER != 0) { WaitVsync(); }
}

// handle the B button being pressed
void private_HandleBButton() {
	u8 temp;
	u8 eventHandled;
	u16 temp2;
	eventHandled = 0;
	
	if( lastEvent == EVENT_CASINO_GAMBLE ) {
		if( money >= 25 ) {
			temp2 = GetGambleResult(25);
			money = money - 25;
			if( temp2 == 0 ) {
				AddMessage( (char*)"You lost!" );
			} else {
				AddMessage( (char*)"You won" );
				AddAmountMessage( temp2 );
				money = money + temp2;
			}
			AddMessageBlank();
			
			// player can keep gambling
			private_GenerateEvent();
			private_RenderEvent();
			eventHandled = 1;
		}
	}
	
	// debounce if we actually did something
	if( eventHandled ) 
		while (SYS_LEVER != 0) { WaitVsync(); }
}

void private_ChangeLevel( u8 which, u8 generateEvents ) {
	FillScreen(SCR_1_PLANE, (u16)' ', 0);
	FillScreen(SCR_2_PLANE, (u16)' ', 0);
	LoadLevel( Levels[which] );
	
	// do we need to generate an event (have we just entered this screen)?
	if( generateEvents ) {
		// we render random events just ONCE, upon entering a level
		if( Levels[currentLevelIndex] != LEVEL_FELLOW_COLLECTOR ) {
			private_GenerateRandomEvent();
		}
		// level-dependent events
		private_GenerateEvent();
	} 
	
	private_RenderEvent();
}


// Copies tiles from the tile data file
// into the tiles on which the four sprites (which show 
// the main character) are based
void private_LoadAnimationTiles(u8 animationFrame) {

	u8 i;
	u16 * TileRam = TILE_RAM;
	u16 * theSpriteAnim = (u16 *) Sprite_Tiles;

	TileRam += (FIRST_SPRITE_TILE * 8);
	theSpriteAnim += (animationFrame * 8);

	for (i = 0; i < 32; i ++) {
		TileRam[i] = *theSpriteAnim++;
	}
}

void private_ShowPlayerSprite( u8 x, u8 y, u8 facing ) {
	u8 i;

	// set the initial sprite up
	// Note that all 4 sprites are chained together so we only have to
	// move the first one and the rest follow :-)
	SetSprite(FIRST_PLAYER_SPRITE, FIRST_SPRITE_TILE, 0, x, y, PLAYER_PALETTE_TOP);
	SetSprite(FIRST_PLAYER_SPRITE+1, FIRST_SPRITE_TILE+1, 1, 8, 0, PLAYER_PALETTE_TOP);
	SetSprite(FIRST_PLAYER_SPRITE+2, FIRST_SPRITE_TILE+2, 1, -8, 8, PLAYER_PALETTE);
	SetSprite(FIRST_PLAYER_SPRITE+3, FIRST_SPRITE_TILE+3, 1, 8, 0, PLAYER_PALETTE);
	
	if( facing == 0 ) {
		private_LoadAnimationTiles(0);
	} else {
		private_LoadAnimationTiles(8);
	}

	// put the sprites between planes
	for (i=FIRST_PLAYER_SPRITE; i<FIRST_PLAYER_SPRITE+4; i++)
		SpriteControl(i, SPR_MIDDLE, 0);
}

void private_ShowNPCSprite() {
	u8 i;
	u8 x, y;
	x = 70; y = 10;
	// set the initial sprite up
	// Note that all 4 sprites are chained together so we only have to
	// move the first one and the rest follow :-)
	SetSprite(FIRST_NPC_SPRITE, FIRST_NPC_TILE, 0, x, y, NPC_PALETTE);
	SetSprite(FIRST_NPC_SPRITE+1, FIRST_NPC_TILE+1, FIRST_NPC_TILE, 8, 0, NPC_PALETTE);
	SetSprite(FIRST_NPC_SPRITE+2, FIRST_NPC_TILE+2, FIRST_NPC_TILE+1, -8, 8, NPC_PALETTE);
	SetSprite(FIRST_NPC_SPRITE+3, FIRST_NPC_TILE+3, FIRST_NPC_TILE+2, 8, 0, NPC_PALETTE);

	// put the sprites between planes
	for (i=FIRST_NPC_SPRITE; i<FIRST_NPC_SPRITE+4; i++)
		SpriteControl(i, SPR_MIDDLE, 0);
}

void MainGame() {

	u8 i;
	
	SetPalette(SPRITE_PLANE, PLAYER_PALETTE_TOP, 0, RGB(12,4,4), RGB(0,0,0), RGB(13,11,10));
	SetPalette(SPRITE_PLANE, PLAYER_PALETTE, 0, RGB(12,4,4), RGB(5,5,14), RGB(13,11,10));
	SetPalette(SPRITE_PLANE, NPC_PALETTE, 0, RGB(3,3,7), RGB(12,0,0), RGB(13,13,10));

	SetPalette(SCR_1_PLANE, 0, 0, RGB(15,0,0), RGB(14,12,12), RGB(0,0,0));
	SetPalette(SCR_1_PLANE, 1, 0, RGB(0,0,15), RGB(12,12,14), RGB(0,0,0));
	
	playerX = 72;
	playerY = 48;
	animationFrame = 0;
	currentLevelIndex = 3;
	facing = 0;
	policeChance = 10;
	lastEvent = EVENT_NONE;
	
	money = 100;
	
	//for(i=0;i<GAME_COUNT-1;i++) GameOwn[i]=1;
	//GameOwn[44] = 1;

	private_ShowPlayerSprite( (s8)playerX, (s8)playerY, facing );
	private_ChangeLevel( currentLevelIndex, 1 );

	while(1) {
		private_PrintMoney();
		
		alreadyAdvancedAnimation = 0;
		GetRandom(100);

		// check for left or right movement
		if ((SYS_LEVER & J_LEFT) && ((s16)(playerX-CHARACTER_SPEED) > 0)) {
			playerX -= CHARACTER_SPEED;
			animationFrame ^= 1; //bitwise XOR is used to alternate values between 0 and 1
			alreadyAdvancedAnimation = 1;
			facing = 0;
		} else if ((SYS_LEVER & J_RIGHT) && (playerX+CHARACTER_SPEED < SCRN_W-16)) {
			playerX += CHARACTER_SPEED;
			animationFrame ^= 1;
			alreadyAdvancedAnimation = 1;
			facing = 1;
			GetRandom(100);
		}

		// check for up or down movement
		if ((SYS_LEVER & J_UP) && ((s16)(playerY-CHARACTER_SPEED) > 12)) {
			playerY -= CHARACTER_SPEED;
			if( !alreadyAdvancedAnimation ) {
				animationFrame ^= 1;
				alreadyAdvancedAnimation = 1;
				}
		} else if ((SYS_LEVER & J_DOWN) && (playerY+CHARACTER_SPEED < SCRN_H-11*8)) {
			playerY += CHARACTER_SPEED;
			if( !alreadyAdvancedAnimation ) {
				animationFrame ^= 1;
				alreadyAdvancedAnimation = 1;
			}
		}

		if( alreadyAdvancedAnimation ) {
			if( facing == 0 ) {
				private_LoadAnimationTiles(0+(animationFrame * 4));
			} else {
				private_LoadAnimationTiles(8+(animationFrame * 4));
			}
			SetSpritePosition(FIRST_PLAYER_SPRITE, (u8)playerX, (u8)playerY);
		}
		
		// do we need to change level because the player was near the edge
		// of the screen?
		if( playerX > SCRN_W-16-8 && currentLevelIndex < MAX_LEVELS-1 ) {
			currentLevelIndex++;
			
			// if we're in the fellow collector screen, show him
			if( Levels[ currentLevelIndex ] == LEVEL_FELLOW_COLLECTOR ) {
				private_ShowNPCSprite();
			} else {
				// else hide him
				for (i=0; i<4; i++)
					UnsetSprite(FIRST_NPC_SPRITE+i);
			}
			
			playerX = 10;
			SetSpritePosition(FIRST_PLAYER_SPRITE, (u8)playerX, (u8)playerY);
			private_ChangeLevel( currentLevelIndex, 1 );
			
			continue;
			
		} else if( playerX < 8 && currentLevelIndex > 0 ) {
			currentLevelIndex--;
			
			// if we're in the fellow collector screen, show him
			if( Levels[ currentLevelIndex ] == LEVEL_FELLOW_COLLECTOR ) {
				private_ShowNPCSprite();
			} else {
				// else hide him
				for (i=0; i<4; i++)
					UnsetSprite(FIRST_NPC_SPRITE+i);
			}
			
			playerX = SCRN_W-16-10;
			SetSpritePosition(FIRST_PLAYER_SPRITE, (u8)playerX, (u8)playerY);
			private_ChangeLevel( currentLevelIndex, 1 );
			
			continue;
		}
		
		Sleep(1);
		
		// was OPTION pressed?
		if ( SYS_LEVER & J_OPTION ) {
			// hide player and NPC
			for (i=0; i<4; i++) {
				UnsetSprite(FIRST_PLAYER_SPRITE+i);
				UnsetSprite(FIRST_NPC_SPRITE+i);
			}
			
			ShowGameList();
			private_ShowPlayerSprite( (s8)playerX, (s8)playerY, facing );
			private_ChangeLevel( currentLevelIndex, 0 );
			if( Levels[ currentLevelIndex ] == LEVEL_FELLOW_COLLECTOR ) {
				private_ShowNPCSprite();
			}
			continue;
		}
		
		// was A pressed?
		if ( SYS_LEVER & J_A ) {
			private_HandleAButton();
			continue;
		}
		
		// was B pressed?
		if ( SYS_LEVER & J_B ) {
			private_HandleBButton();
			continue;
		}
	}
}



